@@ -87,28 +87,30 @@ struct LIBFAUST_API dsp_factory_base {
     
     virtual ~dsp_factory_base() {}
     
-    virtual void write(std::ostream* out, bool binary = false, bool compact = false) {}
+    virtual void write(std::ostream* /*out*/, bool /*binary*/ = false, bool /*compact*/ = false) {}
 };
 
 /**
  *  Print the box.
  *
  * @param box - the box to be printed
- * @param shared - whether the identical sub boxes are printed as indentifier
+ * @param shared - whether the identical sub boxes are printed as identifiers
+ * @param max_size - the maximum number of characters to be printed (possibly needed for big expressions in non shared mode)
  *
  * @return the printed box as a string
  */
-LIBFAUST_API std::string printBox(Box box, bool shared);
+LIBFAUST_API std::string printBox(Box box, bool shared, int max_size);
 
 /**
  *  Print the signal.
  *
  * @param sig - the signal to be printed
- * @param shared - whether the identical sub signals are printed as indentifier
+ * @param shared - whether the identical sub signals are printed as identifiers
+ * @param max_size - the maximum number of characters to be printed (possibly needed for big expressions in non shared mode)
  *
  * @return the printed signal as a string
  */
-LIBFAUST_API std::string printSignal(Signal sig, bool shared);
+LIBFAUST_API std::string printSignal(Signal sig, bool shared, int max_size);
 
 #endif
 
@@ -509,7 +511,6 @@ LIBFAUST_API Box boxXOR(Box b1, Box b2);
 /**
  * Extended unary mathematical functions.
  */
-
 LIBFAUST_API Box boxAbs();
 LIBFAUST_API Box boxAbs(Box x);
 LIBFAUST_API Box boxAcos();
@@ -544,7 +545,6 @@ LIBFAUST_API Box boxAsin(Box x);
 /**
  * Extended binary mathematical functions.
  */
-
 LIBFAUST_API Box boxRemainder();
 LIBFAUST_API Box boxRemainder(Box b1, Box b2);
 LIBFAUST_API Box boxPow();
@@ -730,7 +730,7 @@ LIBFAUST_API tvec boxesToSignals(Box box, std::string& error_msg);
  * @param name_app - the name of the Faust program
  * @param box - the box expression
  * @param lang - the target source code's language which can be one of "c",
- * "cpp", "csharp", "dlang", "fir", "java", "julia", "ocpp", "rust", "soul" or "wast"
+ * "cpp", "cmajor", "csharp", "dlang", "fir", "interp", "java", "julia", "ocpp", "rust" or "wast"
  * (depending of which of the corresponding backends are compiled in libfaust)
  * @param argc - the number of parameters in argv array
  * @param argv - the array of parameters
