@@ -27,6 +27,7 @@
 
 #include <string>
 #include <vector>
+#include <ostream>
 
 #include "faust/export.h"
 
@@ -41,6 +42,40 @@
 class LIBFAUST_API CTree;
 typedef std::vector<CTree*> tvec;
 
+typedef CTree* Signal;
+typedef CTree* Box;
+typedef CTree* Tree;
+
+typedef Tree (*prim0)();
+typedef Tree (*prim1)(Tree x);
+typedef Tree (*prim2)(Tree x, Tree y);
+typedef Tree (*prim3)(Tree x, Tree y, Tree z);
+typedef Tree (*prim4)(Tree w, Tree x, Tree y, Tree z);
+typedef Tree (*prim5)(Tree v, Tree w, Tree x, Tree y, Tree z);
+
+LIBFAUST_API const char* prim0name(prim0);
+LIBFAUST_API const char* prim1name(prim1);
+LIBFAUST_API const char* prim2name(prim2);
+LIBFAUST_API const char* prim3name(prim3);
+LIBFAUST_API const char* prim4name(prim4);
+LIBFAUST_API const char* prim5name(prim5);
+
+/**
+ *  Return the name parameter of a foreign function.
+ *
+ * @param  s - the signal
+ * @return the name
+ */
+LIBFAUST_API const char* ffname(Signal s);
+
+/**
+ *  Return the arity of a foreign function.
+ *
+ * @param  s - the signal
+ * @return the name
+ */
+LIBFAUST_API int ffarity(Signal s);
+
 enum SType { kSInt, kSReal };
 
 enum SOperator { kAdd, kSub, kMul, kDiv, kRem, kLsh, kARsh, kLRsh, kGT, kLT, kGE, kLE, kEQ, kNE, kAND, kOR, kXOR };
@@ -52,19 +87,36 @@ struct LIBFAUST_API dsp_factory_base {
     
     virtual ~dsp_factory_base() {}
     
-    virtual void write(std::ostream* out, bool binary = false, bool compact = false) {}
+    virtual void write(std::ostream* /*out*/, bool /*binary*/ = false, bool /*compact*/ = false) {}
 };
 
+/**
+ *  Print the box.
+ *
+ * @param box - the box to be printed
+ * @param shared - whether the identical sub boxes are printed as identifiers
+ * @param max_size - the maximum number of characters to be printed (possibly needed for big expressions in non shared mode)
+ *
+ * @return the printed box as a string
+ */
+LIBFAUST_API std::string printBox(Box box, bool shared, int max_size);
+
+/**
+ *  Print the signal.
+ *
+ * @param sig - the signal to be printed
+ * @param shared - whether the identical sub signals are printed as identifiers
+ * @param max_size - the maximum number of characters to be printed (possibly needed for big expressions in non shared mode)
+ *
+ * @return the printed signal as a string
+ */
+LIBFAUST_API std::string printSignal(Signal sig, bool shared, int max_size);
+
 #endif
 
 #ifndef LIBFAUSTSIGNAL_H
 #define LIBFAUSTSIGNAL_H
 
-/**
- * Opaque types.
- */
-typedef CTree* Signal;
-
 /**
  * Create global compilation context, has to be done first.
  */
@@ -75,6 +127,51 @@ extern "C" LIBFAUST_API void createLibContext();
  */
 extern "C" LIBFAUST_API void destroyLibContext();
 
+/**
+ * Check if a signal is nil.
+ *
+ * @param s - the signal
+ *
+ * @return true if the signal is nil, otherwise false.
+ */
+LIBFAUST_API bool isNil(Signal s);
+
+/**
+ * Convert a signal (such as the label of a UI) to a string.
+ *
+ * @param s - the signal to convert
+ *
+ * @return a string representation of a signal.
+ */
+LIBFAUST_API const char* tree2str(Signal s);
+
+/**
+ * Return the xtended type of a signal.
+ *
+ * @param s - the signal whose xtended type to return
+ *
+ * @return a pointer to xtended type if it exists, otherwise nullptr.
+ */
+LIBFAUST_API void* getUserData(Signal s);
+
+/**
+ * Return the arity of the xtended signal.
+ *
+ * @param s - the xtended signal
+ *
+ * @return the arity of the xtended signal.
+ */
+LIBFAUST_API unsigned int xtendedArity(Signal s);
+
+/**
+ * Return the name of the xtended signal.
+ *
+ * @param s - the xtended signal
+ *
+ * @return the name of the xtended signal.
+ */
+LIBFAUST_API const char* xtendedName(Signal s);
+
 /**
  * Constant integer : for all t, x(t) = n.
  *
@@ -338,6 +435,25 @@ LIBFAUST_API Signal sigSelf();
  */
 LIBFAUST_API Signal sigRecursion(Signal s);
 
+/**
+ * Create a recursive signal inside the sigRecursionN expression.
+ *
+ * @param id - the recursive signal index (starting from 0, up to the number of outputs signals in the recursive block)
+ *
+ * @return the recursive signal.
+ */
+LIBFAUST_API Signal sigSelfN(int id);
+
+/**
+ * Create a recursive block of signals. Use sigSelfN() to refer to the
+ * recursive signal inside the sigRecursionN expression.
+ *
+ * @param rf - the list of signals to recurse on.
+ *
+ * @return the list of signals with recursions.
+ */
+LIBFAUST_API tvec sigRecursionN(const tvec& rf);
+
 /**
  * Create a button signal.
  *
@@ -434,19 +550,107 @@ LIBFAUST_API Signal sigHBargraph(const std::string& label, Signal min, Signal ma
 LIBFAUST_API Signal sigAttach(Signal s1, Signal s2);
 
 /**
- * Create a C++ Faust DSP factory from a vector of output signals.
+ *  Simplify a signal to its normal form, where:
+ *  - all possible optimisations, simplications, and compile time computations have been done
+ *  - the mathematical functions (primitives and binary functions), delay, select2, sounfile primitive...
+ *  are properly typed (arguments and result)
+ *  - signal cast are properly done when needed
+ *
+ * @param sig - the signal to be processed
+ *
+ * @return the signal in normal form.
+ */
+LIBFAUST_API Signal simplifyToNormalForm(Signal s);
+
+/**
+ *  Simplify a signal vector to its normal form, where:
+ *  - all possible optimisations, simplications, and compile time computations have been done
+ *  - the mathematical functions (primitives and binary functions), delay, select2, sounfile primitive...
+ *  are properly typed (arguments and result)
+ *  - signal cast are properly done when needed
+ *
+ * @param siglist - the signal vector to be processed
+ *
+ * @return the signal vector in normal form.
+ */
+LIBFAUST_API tvec simplifyToNormalForm2(tvec siglist);
+
+/**
+ * Create source code in a target language from a vector of output signals.
  *
  * @param name_app - the name of the Faust program
- * @param signals_vec - the vector of output signals
+ * @param osigs - the vector of output signals (that will internally be converted in normal form,
+ * see simplifyToNormalForm)
+ * @param lang - the target source code's language which can be one of "c",
+ * "cpp", "csharp", "dlang", "fir", "java", "julia", "ocpp", "rust", "soul" or "wast"
+ * (depending of which of the corresponding backends are compiled in libfaust)
  * @param argc - the number of parameters in argv array
  * @param argv - the array of parameters
  * @param error_msg - the error string to be filled
  *
- * @return a DSP factory on success, otherwise a null pointer.
+ * @return a string of source code on success, setting error_msg on error.
  */
-LIBFAUST_API dsp_factory_base* createCPPDSPFactoryFromSignals(const std::string& name_app, tvec signals_vec,
-                                                              int argc, const char* argv[],
-                                                              std::string& error_msg);
+LIBFAUST_API std::string createSourceFromSignals(const std::string& name_app, tvec osigs,
+                                                const std::string& lang,
+                                                int argc, const char* argv[],
+                                                std::string& error_msg);
+
+
+/**
+ * Test each signal and fill additional signal specific parameters.
+ *
+ * @return true and fill the specific parameters if the signal is of a given type, false otherwise
+ */
+LIBFAUST_API bool isSigInt(Signal t, int* i);
+LIBFAUST_API bool isSigReal(Signal t, double* r);
+LIBFAUST_API bool isSigInput(Signal t, int* i);
+LIBFAUST_API bool isSigOutput(Signal t, int* i, Signal& t0);
+LIBFAUST_API bool isSigDelay1(Signal t, Signal& t0);
+LIBFAUST_API bool isSigDelay(Signal t, Signal& t0, Signal& t1);
+LIBFAUST_API bool isSigPrefix(Signal t, Signal& t0, Signal& t1);
+LIBFAUST_API bool isSigRDTbl(Signal s, Signal& t, Signal& i);
+LIBFAUST_API bool isSigWRTbl(Signal u, Signal& id, Signal& t, Signal& i, Signal& s);
+LIBFAUST_API bool isSigGen(Signal t, Signal& x);
+LIBFAUST_API bool isSigDocConstantTbl(Signal t, Signal& n, Signal& sig);
+LIBFAUST_API bool isSigDocWriteTbl(Signal t, Signal& n, Signal& sig, Signal& widx, Signal& wsig);
+LIBFAUST_API bool isSigDocAccessTbl(Signal t, Signal& tbl, Signal& ridx);
+LIBFAUST_API bool isSigSelect2(Signal t, Signal& selector, Signal& s1, Signal& s2);
+LIBFAUST_API bool isSigAssertBounds(Signal t, Signal& s1, Signal& s2, Signal& s3);
+LIBFAUST_API bool isSigHighest(Signal t, Signal& s);
+LIBFAUST_API bool isSigLowest(Signal t, Signal& s);
+
+LIBFAUST_API bool isSigBinOp(Signal s, int* op, Signal& x, Signal& y);
+LIBFAUST_API bool isSigFFun(Signal s, Signal& ff, Signal& largs);
+LIBFAUST_API bool isSigFConst(Signal s, Signal& type, Signal& name, Signal& file);
+LIBFAUST_API bool isSigFVar(Signal s, Signal& type, Signal& name, Signal& file);
+
+LIBFAUST_API bool isProj(Signal s, int* i, Signal& rgroup);
+LIBFAUST_API bool isRec(Signal s, Signal& var, Signal& body);
+
+LIBFAUST_API bool isSigIntCast(Signal s, Signal& x);
+LIBFAUST_API bool isSigFloatCast(Signal s, Signal& x);
+
+LIBFAUST_API bool isSigButton(Signal s, Signal& lbl);
+LIBFAUST_API bool isSigCheckbox(Signal s, Signal& lbl);
+
+LIBFAUST_API bool isSigWaveform(Signal s);
+
+LIBFAUST_API bool isSigHSlider(Signal s, Signal& lbl, Signal& init, Signal& min, Signal& max, Signal& step);
+LIBFAUST_API bool isSigVSlider(Signal s, Signal& lbl, Signal& init, Signal& min, Signal& max, Signal& step);
+LIBFAUST_API bool isSigNumEntry(Signal s, Signal& lbl, Signal& init, Signal& min, Signal& max, Signal& step);
+
+LIBFAUST_API bool isSigHBargraph(Signal s, Signal& lbl, Signal& min, Signal& max, Signal& x);
+LIBFAUST_API bool isSigVBargraph(Signal s, Signal& lbl, Signal& min, Signal& max, Signal& x);
+
+LIBFAUST_API bool isSigAttach(Signal s, Signal& s0, Signal& s1);
+
+LIBFAUST_API bool isSigEnable(Signal s, Signal& s0, Signal& s1);
+LIBFAUST_API bool isSigControl(Signal s, Signal& s0, Signal& s1);
+
+LIBFAUST_API bool isSigSoundfile(Signal s, Signal& label);
+LIBFAUST_API bool isSigSoundfileLength(Signal s, Signal& sf, Signal& part);
+LIBFAUST_API bool isSigSoundfileRate(Signal s, Signal& sf, Signal& part);
+LIBFAUST_API bool isSigSoundfileBuffer(Signal s, Signal& sf, Signal& chan, Signal& part, Signal& ridx);
 
 /*
  [1] Constant numerical expression : see https://faustdoc.grame.fr/manual/syntax/#constant-numerical-expressions
